<?php
/**
 * @file functions.inc
 * additional functions for iserv-mod-room-reservation
 * @author Roland Hieber (roland.hieber@wilhelm-gym.net)
 * @date 22.12.2007
 * 
 * Copyright © 2007 Roland Hieber
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 * @todo move some functions into M Class Library
 */

require_once("share.inc");
require_once("db.inc");
require_once("user.inc");
require_once("sec/secure.inc");

/**
 * @page errorcodes Error Codes
 * @{
 */
/** Access denied. This can be due to missing access rights. */
define("MOD_ROOM_RESERVATION_ERROR_ACCESS_DENIED", 
  _c("room-reservation:Access denied."));
/** Error while querying the database */
define("MOD_ROOM_RESERVATION_ERROR_SQL", 
  _c("room-reservation:Error while trying to query the database."));
/** Error while trying to open a file */
define("MOD_ROOM_RESERVATION_ERROR_OPEN_FILE",
  _c("room-reservation:Could not open file."));
/** Error while trying to write a file */
define("MOD_ROOM_RESERVATION_ERROR_WRITE_FILE",
  _c("room-reservation:Could not write to file."));
/** Error while trying to lock a file */
define("MOD_ROOM_RESERVATION_ERROR_LOCK_FILE",
  _c("room-reservation:Could not lock file."));
/** Error while trying to unlock a file */
define("MOD_ROOM_RESERVATION_ERROR_UNLOCK_FILE",
  _c("room-reservation:Could not unlock file."));
/** The current booking overlaps with an existing booking */
define("MOD_ROOM_RESERVATION_ERROR_BOOKING_OVERLAPS", _c("room-reservation:".
  "The current booking overlaps with an existing booking."));
/** The booking ends before it begins */
define("MOD_ROOM_RESERVATION_ERROR_END_BEFORE_BEGIN", _c("room-reservation:".
  "The ending of your booking cannot lie before the beginning."));
define("MOD_ROOM_RESERVATION_ERROR_NO_REASON", _c("room-reservation:Please ".
 "give a reason."));
/** A new timeslice overlaps with an existing timeslice */
define("MOD_ROOM_RESERVATION_ERROR_CONFIG_OVERLAPPING_TIMESLICE",
  _c("room-reservation:The period overlaps with an existing one."));
/** There is no such timeslice */
define("MOD_ROOM_RESERVATION_ERROR_CONFIG_NO_SUCH_TIMESLICE",
  _c("room-reservation:The specified period does not exist."));
/** There is no such account */
define("MOD_ROOM_RESERVATION_ERROR_NO_SUCH_ACCOUNT", _c("room-reservation:".
  "The specified account does not exist."));
/** The room is not available for booking */
define("MOD_ROOM_RESERVATION_ERROR_ROOM_NOT_WHITELISTED",
  _c("room-reservation:This room is not available for booking."));
/**
 * @}
 */

/**
 * Determine if the specified user exists. Throws an SQLException if an error
 * occured.
 * @param $strAct (string) Account name of the user
 * @throws SQLException
 * @return (bool)
 */
function isAct($strAct) {
  $hQuery = db_query("SELECT * FROM users WHERE act = $1;", $strAct);
  if(!is_resource($hQuery)) {
    throw new SQLException(MOD_ROOM_RESERVATION_ERROR_SQL);
    return null;
  }
  return (pg_num_rows($hQuery) > 0);
}

/**
 * Get the real user name for an account name
 * @param $strAct (string) Account name of the user to look up
 * @return (string) The real name of the user. If the function fails, it returns <tt>null</tt>.
 *  Call getLastError() to get more information about the error.
 */
function getRealUserName($strAct) {
  $hQuery = db_query("SELECT firstname, lastname FROM users WHERE act = $1;", $strAct);
  if(!is_resource($hQuery)) {
    // TODO throw exception
    setLastError(RS_ERROR_SQL);
    return null;
  }
  if(pg_num_rows($hQuery) == 0) {
    return $strAct;     // User not found in database, return account name
  }
  $arResult = pg_fetch_array($hQuery);
  return user_join_name($arResult);
}

/**
 * Determine if a specified group exists
 * @param $strAct (string) Account name of the group
 * @return  (bool / null) If the function fails, it returns <tt>null</tt>. Call getLastError() to
 *  get more information about the error.
 */
function isGroup($strAct) {
  $hQuery = db_query("SELECT * FROM groups WHERE act = $1;", $strAct);
  if(!is_resource($hQuery)) {
    // TODO throw exception
    setLastError(RS_ERROR_SQL);
    return null;
  }
  return (pg_num_rows($hQuery) > 0);
}

/**
 * Look up the name of a group
 * @param $strAct (string) Account name of the group
 * @return (string) The name of the group. If the function fails, it returns <tt>null</tt>.
 *  Call getLastError() to get more information about the error.
 */
function getGroupName($strAct) {
  $hQuery = db_query("SELECT * FROM groups WHERE act = $1;", $strAct);
  if(!is_resource($hQuery)) {
    // TODO throw exception
    setLastError(RS_ERROR_SQL);
    return null;
  }
  if(pg_num_rows($hQuery) == 0) {
    return $strAct;     // Group not found in database, return account name
  }
  $arResult = pg_fetch_array($hQuery);
  return $arResult["name"];
}

/**
 * Create a link to write a mail to the specified account name.
 * This function returns a link if the specified account exists, otherwise it returns the
 * account name.
 * @param $strAct (string) Account name
 * @param $strColor (string) Background color for icon()
 * @param $strParams (string) additional URL parameters
 * @return string
 */
function mailToUserLink($strAct, $strColor = "bl", $strParams = "") {
  if(!isAct($strAct)) {
    return $strAct;
  }
  return popup(relroot("msg/write/?to=".user_mail_addr($strAct).$strParams),
    600, 400, nobr(icon("mail-reply-usr", array("size" => 16, "bg" => 
    $strColor)) . getRealUserName($strAct)));
}

/**
 * Determine if a specified string is a valid mail address
 * @param $strAddr string
 * @return string
 */
function isMailAddress($strAddr) {
  return ((preg_match("/([a-zA-Z0-9_\-\.]*(@[a-zA-Z0-9\-\.]*)?(\s*,\s*)?)+/", $strAddr) > 0)
  and (preg_match("/(\s*,\s*)$/", $strAddr) == 0));
}

/**
 * Module-specific logging function.
 * Prefixes the log message with a module-specific string and writes it to the logs.
 * @param $strLog (string) Log message
 * @return void
 */
function rrInsertLog($strLog) {
 log_insert($strLog, null, "Room Reservation Schedule");
}

/**
 * Get the SQL day number from a given timestamp.
 * @param $ts (timestamp)
 * @return (int) 0-6 for Sunday to Monday
 */
function rrDateToSQLDayNumber($ts) {
  $aDays = array("Sun" => 0, "Mon" => 1, "Tue" => 2,
    "Wed" => 3, "Thu" => 4, "Fri" => 5, "Sat" => 6);
  return $aDays[date("D", $ts)];
}

/**
 * Convert a UNIX timestamp to an SQL date string
 * @param $ts (timestamp)
 * @return string
 */
function dateToSql($ts) { return date("Y\-m\-d", intval($ts)); }

/**
 * Calculate the timestamp of the monday in the current week
 * @param $ts (timestamp) Calculate the value relative to this date
 * @return timestamp
 */
function rrGetMonday($ts = null) {
  if($ts === null) {
    $ts = time();
  }
  if(date("D", $ts) == "Mon") {
    // Today is monday
    return strtotime("00:00", $ts);
  } else {
    return strtotime("last monday", $ts);
  }
}

/** (array of strings) Additional CSS rules */
$GLOBALS["rrLocalCss"] = array();

/**
 * Add CSS rules to the page
 * @param $s (string)
 * @return void
 */
function rrAddCss($s) {
  rrDebug("rrAddCss: add \"$s\"");
  $GLOBALS["rrLocalCss"][] = $s;
}

/**
 * Get CSS rules that have been added with rrAddCss()
 * @return string
 */
function rrGetCss() {
  rrDebug("rrGetCss: Local CSS is ".var_export($GLOBALS["rrLocalCss"], true));
  return implode("\n", $GLOBALS["rrLocalCss"]);
}

function rrDebug($s, $bReturn = false) {
  if(isset($_GET["debug"])) {
    if(!$bReturn) {
      echo "<!-- $s -->\n";
    } else {
      return $s;
    }
  }
}

/**
 * sprintf with support for ordinal numbers.
 * This version of sprintf replaces all substrings of the type <tt>/\\d+#/</tt>
 * (i.e. a decimal number with a hash sign appended) in the input string with
 * the corresponding english ordinal number prefices (st, nd, rd, th).
 * @param $str (string) Input string
 * @param $args (mixed) Additional parameters to be passed to sprintf()
 * @return (string)
 */
function _sprintf_ord($str, $args /*leave this parameters for doxygen*/) {
  $args = func_get_args();
  if(preg_match_all("/%[bcdufosxX]/", $args[0], $temp) != func_num_args()-1) {
    trigger_error("Too few arguments", E_USER_ERROR);
    return false;
  }
  $str = call_user_func_array("sprintf", $args);
  while(preg_match("/(.*)(\d+)#(.*)/", $str, $m))
    $str = $m[1]._(append_ord_suffix($m[2])).$m[3];
  return $str;
}
?>
